package it.inaf.oats.ia2.tapschemamanager.businesslayer;

import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaEntity;
import java.io.Serializable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 * @param <T>
 */
public abstract class EntityWrapper<T extends TapSchemaEntity> implements Serializable {

    private static final long serialVersionUID = -8399058826120891216L;

    private final T entity;
    private final Map<String, String> originalValues;
    private final Map<String, String> editedValues;
    private final List<String> supportedKeys;
    private Status status;

    public EntityWrapper(T entity, String... supportedKeys) {
        originalValues = new HashMap<String, String>();
        editedValues = new HashMap<String, String>();
        this.entity = entity;
        this.supportedKeys = Arrays.asList(supportedKeys);
    }

    public List<String> getSupportedKeys() {
        return supportedKeys;
    }

    protected final void addValue(String key, String value) {
        originalValues.put(key, value);
    }

    public final String getValue(String key) {
        String editedValue = editedValues.get(key);
        if (editedValue != null) {
            return editedValue;
        }
        return originalValues.get(key);
    }

    protected abstract void afterSetValue(String key, String value);

    public final void setValue(String key, String value) {
        if (originalValues.get(key) == null && value.equals("")) {
            value = null;
        }
        editedValues.put(key, value);
        afterSetValue(key, value);
    }

    public final boolean isChanged(String key) {
        String originalValue = originalValues.get(key);
        String editedValue = editedValues.get(key);
        if (originalValue == null) {
            return editedValue != null;
        } else {
            if (editedValue != null && !originalValue.equals(editedValue)) {
                return true;
            }
        }
        return false;
    }

    public final boolean isChanged() {
        for (String key : editedValues.keySet()) {
            if (isChanged(key)) {
                return true;
            }
        }
        return false;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    public T getEntity() {
        return entity;
    }

    public String getName() {
        return entity.getName();
    }

    protected void setPersisted() {
        setStatus(Status.ADDED_PERSISTED);
        for (String key : editedValues.keySet()) {
            originalValues.put(key, editedValues.get(key));
        }
        editedValues.clear();
    }

    protected void undoChanges() {
        editedValues.clear();
    }

    public boolean isToRemove() {
        return status == Status.TO_REMOVE;
    }
}
