package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import javax.xml.bind.annotation.XmlAttribute;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class Credentials implements Serializable {

    private static final long serialVersionUID = 1153912575502196261L;
    private static final Logger log = LoggerFactory.getLogger(Credentials.class);

    private String hostname;
    private int port;
    private String username;
    private String password;
    private String database;

    private DatabaseType databaseType;

    public Credentials() {
        this(DatabaseType.MYSQL);
    }

    public Credentials(DatabaseType dbType) {
        this.databaseType = dbType;
        this.setDefaults();
    }

    @XmlAttribute(name = "hostname", required = true)
    public String getHostname() {
        return this.hostname;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    @XmlAttribute(name = "port", required = true)
    public int getPort() {
        return this.port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    @XmlAttribute(name = "username", required = true)
    public String getUsername() {
        return this.username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    @XmlAttribute(name = "password", required = true)
    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    @XmlAttribute(name = "database-type", required = true)
    public DatabaseType getDatabaseType() {
        return databaseType;
    }

    public void setDatabaseType(DatabaseType databaseType) {
        this.databaseType = databaseType;
    }

    /**
     * Only for POSTGRES
     */
    @XmlAttribute(name = "database")
    public String getDatabase() {
        return database;
    }

    /**
     * Only for POSTGRES
     */
    public void setDatabase(String database) {
        this.database = database;
    }

    public final void setDefaults() {
        if (databaseType == DatabaseType.MYSQL) {
            this.port = 3306;
            this.database = null;
        } else if (databaseType == DatabaseType.POSTGRES) {
            this.port = 5432;
            this.database = "postgres";
        }
    }

    public String getDatabaseDriverClass() {
        switch (getDatabaseType()) {
            case MYSQL:
                return "com.mysql.jdbc.Driver";
            case POSTGRES:
                return "org.postgresql.Driver";
        }
        return null;
    }

    public Connection getConnection() throws SQLException {
        try {
            Class.forName(getDatabaseDriverClass());
        } catch (ClassNotFoundException e) {
            e.printStackTrace(System.err);
        }

        String connectionString = "jdbc:mysql://" + this.hostname + ":" + this.port + "/";
        log.debug("Connecting to {} with username '{}'", connectionString, this.username);
        return DriverManager.getConnection(connectionString, this.username, this.password);
    }
}
