package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.MapKeyColumn;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Entity
@Table(name = "\"schemas\"")
public class SchemaEntity implements Serializable, TapSchemaEntity {
    
    private static final long serialVersionUID = -1058919664128696098L;
    
    @Id
    @Column(name = "schema_name", length = 64)
    private String name;
    
    @Column(name = "utype", length = 512)
    private String utype;
    
    @Column(name = "description", length = 512)
    private String description;
    
    @Column(name = "schemaID")
    private BigInteger schemaID;
    
    @OneToMany(mappedBy = "schema", cascade = CascadeType.ALL, orphanRemoval = true)
    @MapKeyColumn(name = "table_name", insertable = false, updatable = false)
    @OrderBy("name ASC")
    private final List<TableEntity> tables;
    
    protected SchemaEntity() {
        this.tables = new ArrayList<TableEntity>();
    }
    
    protected SchemaEntity(String schemaName) {
        this();
        this.name = schemaName;
    }
    
    @Override
    public String getName() {
        return name;
    }
    
    protected void setName(String schemaName) {
        this.name = schemaName;
    }
    
    public String getUtype() {
        return utype;
    }
    
    public void setUtype(String utype) {
        this.utype = utype;
    }
    
    public String getDescription() {
        return description;
    }
    
    public void setDescription(String description) {
        this.description = description;
    }
    
    protected BigInteger getSchemaID() {
        return schemaID;
    }
    
    protected void setSchemaID(BigInteger schemaID) {
        this.schemaID = schemaID;
    }
    
    public List<TableEntity> getTables() {
        return Collections.unmodifiableList(tables);
    }
    
    public TableEntity getTableByShortName(String shortTableName) {
        return getTable(name + "." + shortTableName);
    }
    
    public TableEntity getTable(String fullTableName) {
        return DLUtil.getEntityByName(tables, fullTableName);
    }
    
    protected void addTable(TableEntity table) {
        DLUtil.insertEntityInOrder(tables, table);
        table.setSchemaName(name);
        table.setSchema(this);
    }
    
    protected void removeTable(String fullTableName) {
        DLUtil.removeEntity(tables, fullTableName);
    }
}
